from typing import Literal

import numpy as np
import numpy.typing as npt
from scipy._typing import Untyped, UntypedArray

__all__ = [
    "choose_conv_method",
    "cmplx_sort",
    "convolve",
    "convolve2d",
    "correlate",
    "correlate2d",
    "correlation_lags",
    "decimate",
    "deconvolve",
    "detrend",
    "fftconvolve",
    "filtfilt",
    "hilbert",
    "hilbert2",
    "invres",
    "invresz",
    "lfilter",
    "lfilter_zi",
    "lfiltic",
    "medfilt",
    "medfilt2d",
    "oaconvolve",
    "order_filter",
    "resample",
    "resample_poly",
    "residue",
    "residuez",
    "sosfilt",
    "sosfilt_zi",
    "sosfiltfilt",
    "unique_roots",
    "vectorstrength",
    "wiener",
]

def correlate(in1, in2, mode: str = "full", method: str = "auto") -> Untyped: ...
def correlation_lags(in1_len, in2_len, mode: str = "full") -> Untyped: ...
def fftconvolve(in1, in2, mode: str = "full", axes: Untyped | None = None) -> Untyped: ...
def oaconvolve(in1, in2, mode: str = "full", axes: Untyped | None = None) -> Untyped: ...
def choose_conv_method(in1, in2, mode: str = "full", measure: bool = False) -> Untyped: ...
def convolve(in1, in2, mode: str = "full", method: str = "auto") -> Untyped: ...
def order_filter(a, domain, rank) -> Untyped: ...
def medfilt(volume, kernel_size: Untyped | None = None) -> Untyped: ...
def wiener(im, mysize: Untyped | None = None, noise: Untyped | None = None) -> Untyped: ...
def convolve2d(in1, in2, mode: str = "full", boundary: str = "fill", fillvalue: int = 0) -> Untyped: ...
def correlate2d(in1, in2, mode: str = "full", boundary: str = "fill", fillvalue: int = 0) -> Untyped: ...
def medfilt2d(input, kernel_size: int = 3) -> Untyped: ...
def lfilter(b, a, x, axis: int = -1, zi: Untyped | None = None) -> Untyped: ...
def lfiltic(b, a, y, x: Untyped | None = None) -> Untyped: ...
def deconvolve(signal, divisor) -> Untyped: ...
def hilbert(x, N: Untyped | None = None, axis: int = -1) -> Untyped: ...
def hilbert2(x, N: Untyped | None = None) -> Untyped: ...
def unique_roots(p, tol: float = 0.001, rtype: str = "min") -> Untyped: ...
def invres(r, p, k, tol: float = 0.001, rtype: str = "avg") -> Untyped: ...
def residue(b, a, tol: float = 0.001, rtype: str = "avg") -> Untyped: ...
def residuez(b, a, tol: float = 0.001, rtype: str = "avg") -> Untyped: ...
def invresz(r, p, k, tol: float = 0.001, rtype: str = "avg") -> Untyped: ...
def resample(x, num, t: Untyped | None = None, axis: int = 0, window: Untyped | None = None, domain: str = "time") -> Untyped: ...
def resample_poly(
    x, up, down, axis: int = 0, window=("kaiser", 5.0), padtype: str = "constant", cval: Untyped | None = None
) -> Untyped: ...
def vectorstrength(events, period) -> Untyped: ...
def detrend(
    data: UntypedArray,
    axis: int = -1,
    type: Literal["linear", "constant"] = "linear",
    bp: npt.ArrayLike | int = 0,
    overwrite_data: bool = False,
) -> UntypedArray: ...
def lfilter_zi(b, a) -> Untyped: ...
def sosfilt_zi(sos) -> Untyped: ...
def filtfilt(
    b,
    a,
    x,
    axis: int = -1,
    padtype: str = "odd",
    padlen: Untyped | None = None,
    method: str = "pad",
    irlen: Untyped | None = None,
) -> Untyped: ...
def sosfilt(sos, x, axis: int = -1, zi: Untyped | None = None) -> Untyped: ...
def sosfiltfilt(sos, x, axis: int = -1, padtype: str = "odd", padlen: Untyped | None = None) -> Untyped: ...
def decimate(x, q, n: Untyped | None = None, ftype: str = "iir", axis: int = -1, zero_phase: bool = True) -> Untyped: ...
